/*
 * $Id: InboundMessageLossTransactionsTestCase.java 22018 2011-05-27 20:55:34Z tcarlson $
 * --------------------------------------------------------------------------------------
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 *
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.transport.jms.reliability;

import java.util.concurrent.TimeUnit;


/**
 * Verify that no inbound messages are lost when exceptions occur.  
 * The message must either make it all the way to the SEDA queue (in the case of 
 * an asynchronous inbound endpoint), or be restored/rolled back at the source.
 * 
 * In the case of JMS, this will cause the failed message to be redelivered if 
 * JMSRedelivery is configured.
 */
public class InboundMessageLossTransactionsTestCase extends InboundMessageLossTestCase
{
    @Override
    protected String getConfigResources()
    {
        return "reliability/activemq-config.xml, reliability/inbound-message-loss-transactions.xml";
    }
    
    @Override
    public void testComponentException() throws Exception
    {
        putMessageOnQueue("componentException");
        
        // Although a component exception occurs after the SEDA queue, the use of transactions 
        // bypasses the SEDA queue, so message should get redelivered.
        assertTrue("Message should have been redelivered", 
            messageRedelivered.await(latchTimeout, TimeUnit.MILLISECONDS));
    }    
}
